from fastapi import APIRouter, Depends, HTTPException, UploadFile, File, Form
from sqlmodel import Session, func
from geopy.geocoders import Nominatim
from backend.models.models import Shop, User, Order
from backend.schemas.shop import ShopRead
from backend.database import get_session
from backend.routes.auth import get_current_user
from core.config import settings
import shutil
import os

router = APIRouter()

# Define your backend host URL.
BACKEND_HOST = "http://127.0.0.1:8000"  # Or use settings.backend_url if available

@router.post("/create", response_model=ShopRead)
def create_shop(
    name: str = Form(...),
    description: str = Form(None),
    address: str = Form(...),
    file: UploadFile = File(None),
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    # Get latitude and longitude from address
    geolocator = Nominatim(user_agent="shop_locator")
    location = geolocator.geocode(address)
    if not location:
        raise HTTPException(status_code=400, detail="Invalid address")

    shop = Shop(
        name=name,
        description=description,
        address=address,
        latitude=location.latitude,
        longitude=location.longitude,
        owner_id=current_user.id,
    )
    session.add(shop)
    session.commit()
    session.refresh(shop)

    # Create a folder in the static directory for the shop
    shop_dir = os.path.join(settings.static_dir, f"shop_{shop.name}")
    os.makedirs(shop_dir, exist_ok=True)

    if file and file.filename:
        file_location = os.path.join(shop_dir, file.filename)
        with open(file_location, "wb") as buffer:
            shutil.copyfileobj(file.file, buffer)
        relative_path = os.path.relpath(file_location, settings.static_dir)
        relative_path = relative_path.replace("\\", "/")
        public_url = f"{BACKEND_HOST}/static/{relative_path}"
        shop.image_url = public_url
    else:
        # Use default shop image from static/default folder
        default_path = os.path.join(settings.static_dir, "default", "default_shop.png")
        relative_path = os.path.relpath(default_path, settings.static_dir)
        relative_path = relative_path.replace("\\", "/")
        public_url = f"{BACKEND_HOST}/static/{relative_path}"
        shop.image_url = public_url

    session.commit()
    session.refresh(shop)
    return shop


@router.get("/list", response_model=list[ShopRead])
def get_all_shops(order: str = "desc", session: Session = Depends(get_session)):
    order_by = (
        func.count(Order.id).desc() if order == "desc" else func.count(Order.id).asc()
    )
    shops = (
        session.query(Shop)
        .outerjoin(Order, Shop.id == Order.shop_id)
        .group_by(Shop.id)
        .order_by(order_by)
        .all()
    )
    return shops


@router.get("/get/{shop_id}", response_model=ShopRead)
def read_shop(shop_id: int, session: Session = Depends(get_session)):
    shop = session.get(Shop, shop_id)
    if not shop:
        raise HTTPException(status_code=404, detail="Shop not found")
    return shop


@router.put("/put/{shop_id}", response_model=ShopRead)
def update_shop(
    shop_id: int,
    name: str = Form(None),
    description: str = Form(None),
    address: str = Form(None),
    file: UploadFile = File(None),
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    shop = session.get(Shop, shop_id)
    if not shop:
        raise HTTPException(status_code=404, detail="Shop not found")
    if shop.owner_id != current_user.id:
        raise HTTPException(status_code=403, detail="Unauthorized to update this shop")
    
    if name:
        shop.name = name
    if description:
        shop.description = description
    if address:
        geolocator = Nominatim(user_agent="shop_locator")
        location = geolocator.geocode(address)
        if not location:
            raise HTTPException(status_code=400, detail="Invalid address")
        shop.address = address
        shop.latitude = location.latitude
        shop.longitude = location.longitude

    shop_dir = os.path.join(settings.static_dir, f"shop_{shop.name}")
    os.makedirs(shop_dir, exist_ok=True)

    if file and file.filename:
        file_location = os.path.join(shop_dir, file.filename)
        with open(file_location, "wb") as buffer:
            shutil.copyfileobj(file.file, buffer)
        relative_path = os.path.relpath(file_location, settings.static_dir)
        relative_path = relative_path.replace("\\", "/")
        public_url = f"{BACKEND_HOST}/static/{relative_path}"
        shop.image_url = public_url
    else:
        default_path = os.path.join(settings.static_dir, "default", "default_shop.png")
        relative_path = os.path.relpath(default_path, settings.static_dir)
        relative_path = relative_path.replace("\\", "/")
        public_url = f"{BACKEND_HOST}/static/{relative_path}"
        shop.image_url = public_url

    session.add(shop)
    session.commit()
    session.refresh(shop)
    return shop


@router.delete("/delete/{shop_id}")
def delete_shop(
    shop_id: int,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    shop = session.get(Shop, shop_id)
    if not shop:
        raise HTTPException(status_code=404, detail="Shop not found")
    if shop.owner_id != current_user.id:
        raise HTTPException(status_code=403, detail="Unauthorized to delete this shop")
    session.delete(shop)
    session.commit()
    return {"message": "Shop deleted successfully"}
