import customtkinter as ctk
import requests
from PIL import Image, ImageTk
from tkinter import messagebox
import io

SHOPPING = "#00c1ff"
BACKEND_HOST = "http://127.0.0.1:8000"  # Adjust if needed

def fix_url(url):
    """
    If the provided URL does not start with http, assume it's a relative path.
    Remove any unwanted prefix (e.g., "app/static/") and prepend the public URL.
    """
    if url.startswith("http"):
        return url
    # If the URL starts with "app/static/", remove that part.
    prefix = "app/static/"
    if url.startswith(prefix):
        url = url[len(prefix):]
    # Prepend the public URL
    return f"{BACKEND_HOST}/static/{url}"

def dashboard_frame(parent, switch_func, API_URL, token):
    """
    Main dashboard UI:
      - Header (top bar) with logo, search, user, cart
      - Middle area with horizontal scroll of top shops
      - Bottom area with grid layout of recommended products
    """
    frame = ctk.CTkFrame(parent, fg_color="transparent")

    # ------------- HEADER (Top Bar) -------------
    header_frame = ctk.CTkFrame(frame, fg_color=SHOPPING)
    header_frame.place(relx=0, rely=0, relwidth=1, relheight=0.1)

    # App Logo (Left)
    logo_label = ctk.CTkLabel(
        header_frame,
        text="Shopping App",
        text_color="white",
        font=("Helvetica", 20, "bold"),
    )
    logo_label.place(relx=0.01, rely=0.1, relwidth=0.15, relheight=0.8)

    # Search bar (Center)
    search_entry = ctk.CTkEntry(
        header_frame, placeholder_text="Search in Shop...", height=30
    )
    search_entry.place(relx=0.25, rely=0.25, relwidth=0.45, relheight=0.5)

    def perform_search():
        """Call an endpoint to search products by keyword."""
        keyword = search_entry.get().strip()
        if not keyword:
            messagebox.showinfo("Info", "Please enter a search keyword.")
            return
        try:
            headers = {"Authorization": f"Bearer {token}"}
            # Adjust endpoint/params as needed
            resp = requests.get(f"{API_URL}/search?name={keyword}&search_type=products", headers=headers)
            if resp.status_code == 200:
                products = resp.json()
                display_products(products, bottom_products_frame)
            else:
                messagebox.showerror("Error", f"Search failed. Status code: {resp.status_code}")
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    search_button = ctk.CTkButton(
        header_frame,
        text="Search",
        fg_color="white",
        text_color="black",
        command=perform_search,
    )
    search_button.place(relx=0.71, rely=0.25, relwidth=0.08, relheight=0.5)

    # ------------- USER & CART ICONS (Top-Right) -------------
    def open_user_details():
        switch_func("user_details")

    def open_cart_details():
        switch_func("user_orders")

    user_button = ctk.CTkButton(
        header_frame,
        text="User",
        fg_color="white",
        text_color="black",
        command=open_user_details,
    )
    user_button.place(relx=0.82, rely=0.25, relwidth=0.08, relheight=0.5)

    cart_button = ctk.CTkButton(
        header_frame,
        text="Cart",
        fg_color="white",
        text_color="black",
        command=open_cart_details,
    )
    cart_button.place(relx=0.91, rely=0.25, relwidth=0.08, relheight=0.5)

    # ------------- MIDDLE (Featured/Top Shops) -------------
    middle_frame = ctk.CTkFrame(frame, fg_color="transparent")
    middle_frame.place(relx=0, rely=0.1, relwidth=1, relheight=0.25)

    featured_label = ctk.CTkLabel(
        middle_frame, text="TOP SHOP", font=("Helvetica", 16, "bold")
    )
    featured_label.pack(pady=5)

    top_shops_frame = ctk.CTkScrollableFrame(
        middle_frame, fg_color="transparent", width=750, height=150
    )
    top_shops_frame.pack(fill="both", expand=True, padx=10, pady=5)

    def create_shop_card(parent, shop_data):
        """
        Create a card for a shop with its image on top and name below.
        """
        card = ctk.CTkFrame(parent, corner_radius=5, fg_color="#2b2b2b", width=100, height=130)
        card.pack_propagate(False)

        image_label = ctk.CTkLabel(card, text="")
        image_label.pack(pady=5)

        # Use "image_url" as returned by the backend, then fix it if needed.
        image_url = shop_data.get("image_url")
        if image_url:
            fixed_url = fix_url(image_url)
            print(f"[DEBUG] Fetching shop image from: {fixed_url}")
            try:
                resp = requests.get(fixed_url)
                print(f"[DEBUG] Status code for shop image: {resp.status_code}")
                if resp.status_code == 200:
                    pil_img = Image.open(io.BytesIO(resp.content)).resize((60, 60))
                    tk_img = ImageTk.PhotoImage(pil_img)
                    image_label.configure(image=tk_img, text="")
                    image_label.image = tk_img
                else:
                    print(f"[DEBUG] Failed to fetch shop image. Status: {resp.status_code}")
            except Exception as e:
                print(f"[DEBUG] Shop image error: {e}")
        else:
            print(f"[DEBUG] No 'image_url' found for shop: {shop_data.get('name')}")

        name_text = shop_data.get("name", "No Name")
        name_label = ctk.CTkLabel(card, text=name_text, font=("Helvetica", 11, "bold"), wraplength=90)
        name_label.pack()

        return card

    def display_shops(shops, container):
        for widget in container.winfo_children():
            widget.destroy()

        if not shops:
            ctk.CTkLabel(container, text="No shops found.").pack(pady=10)
            return

        inner_frame = ctk.CTkFrame(container, fg_color="transparent")
        inner_frame.pack(side="top", fill="both", expand=True)

        for shop in shops:
            shop_card = create_shop_card(inner_frame, shop)
            shop_card.pack(side="left", padx=5, pady=5)

    # ------------- BOTTOM (Products) -------------
    bottom_frame = ctk.CTkFrame(frame, fg_color="transparent")
    bottom_frame.place(relx=0, rely=0.35, relwidth=1, relheight=0.65)

    recommend_label = ctk.CTkLabel(
        bottom_frame, text="TODAY'S RECOMMENDATIONS", font=("Helvetica", 16, "bold")
    )
    recommend_label.pack(pady=5)

    bottom_products_frame = ctk.CTkScrollableFrame(
        bottom_frame, fg_color="transparent", width=750, height=300
    )
    bottom_products_frame.pack(fill="both", expand=True, padx=10, pady=5)

    def create_product_card(parent, product_data):
        card_width = 130
        card_height = 210
        card = ctk.CTkFrame(parent, corner_radius=5, fg_color="#2b2b2b", width=card_width, height=card_height)
        card.pack_propagate(False)

        image_label = ctk.CTkLabel(card, text="")
        image_label.pack(pady=5)

        product_images = product_data.get("images", [])
        if product_images:
            img_url = product_images[0].get("image_url")
            if img_url:
                fixed_url = fix_url(img_url)
                print(f"[DEBUG] Fetching product image from: {fixed_url}")
                try:
                    resp = requests.get(fixed_url)
                    print(f"[DEBUG] Status code for product image: {resp.status_code}")
                    if resp.status_code == 200:
                        pil_img = Image.open(io.BytesIO(resp.content)).resize((70, 70))
                        tk_img = ImageTk.PhotoImage(pil_img)
                        image_label.configure(image=tk_img, text="")
                        image_label.image = tk_img
                    else:
                        print(f"[DEBUG] Failed to fetch product image. Status: {resp.status_code}")
                except Exception as e:
                    print(f"[DEBUG] Product image error: {e}")
        else:
            print(f"[DEBUG] No images found for product: {product_data.get('name')}")

        name_text = product_data.get("name", "No Name")
        name_label = ctk.CTkLabel(card, text=name_text, font=("Helvetica", 11, "bold"), wraplength=120, anchor="center", justify="center")
        name_label.pack(padx=5, pady=3)

        price_val = product_data.get("price", 0.0)
        price_label = ctk.CTkLabel(card, text=f"₫ {price_val:,.1f}", font=("Helvetica", 10, "bold"), text_color="#ff4242")
        price_label.pack(side="bottom", pady=5)

        return card

    def display_products(products, container):
        for widget in container.winfo_children():
            widget.destroy()

        if not products:
            ctk.CTkLabel(container, text="No products found.").pack(pady=10)
            return

        grid_frame = ctk.CTkFrame(container, fg_color="transparent")
        grid_frame.pack(fill="both", expand=True)

        columns = 7  # 7 products per row
        for idx, product in enumerate(products):
            row = idx // columns
            col = idx % columns
            product_card = create_product_card(grid_frame, product)
            product_card.grid(row=row, column=col, padx=5, pady=5)

    def fetch_featured_shops():
        headers = {"Authorization": f"Bearer {token}"}
        try:
            resp = requests.get(f"{API_URL}/shops/list", headers=headers)
            if resp.status_code == 200:
                shops = resp.json()
                display_shops(shops[:5], top_shops_frame)
            else:
                messagebox.showerror("Error", f"Failed to fetch featured shops. Status: {resp.status_code}")
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    def fetch_recommendations():
        headers = {"Authorization": f"Bearer {token}"}
        try:
            resp = requests.get(f"{API_URL}/product/list", headers=headers)
            if resp.status_code == 200:
                products = resp.json()
                display_products(products[:20], bottom_products_frame)
            else:
                messagebox.showerror("Error", f"Failed to fetch recommended products. Status: {resp.status_code}")
        except Exception as e:
            messagebox.showerror("Error", f"Request error: {e}")

    # Load data on start
    fetch_featured_shops()
    fetch_recommendations()

    return frame
